/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use,   *
 * modify, and distribute the following source code (the "Software")     *
 * on any number of computers. You may use the Software as part of       *
 * creating a software program or product intended for commercial or     *
 * non-commercial distribution in machine-readable source code, binary,  *
 * or executable formats. You may distribute the Software as             *
 * machine-readable source code provided this license is not removed     *
 * from the Software and any modifications are conspicuously indicated.  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not remove any copyright, trademark, or other proprietary     *
 * notices from the Software or the associated  documentation.           *
 * You are responsible for compliance with all applicable export or      *
 * re-export control laws and regulations if you export the Software.    *
 * This license is governed by and is to be construed under the laws     *
 * of the State of Texas.                                                *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */

/*
****************************************************************************
**
**  MODULE DESCRIPTION:
**
**      FCNTL - fcntl for OpenVMS
**
**      Implements some of the fcntl functions, and reports the ones that
**      it can't handle.
*/
#include "vms_jackets.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ioctl.h>
#include <fcntl.h>
#include <unistd.h>

/*
** One day these will be in fcntl.h, but until that day...
*/
#if !defined (F_DUPFD)
#define  F_DUPFD        0       /* Duplicate fildes             */
#define  F_GETFD        1       /* Get fildes flags             */
#define  F_SETFD        2       /* Set fildes flags             */
#define  F_GETFL        3       /* Get file flags               */
#define  F_SETFL        4       /* Set file flags               */
#define  F_GETLK        7       /* Get file lock                */
#define  F_SETLK        8       /* Set file lock                */
#define  F_SETLKW       9       /* Set file lock and wait       */
#endif

/*
** The proto in fcntl.h doesn't appear to do the prefix thing.
*/
#define fcntl decc$fcntl
int fcntl   (int __fd, int __cmd, ...);


int GENERIC_FCNTL(int fildes, int cmd, ...)
{
    unsigned int zero=0, one=1, handled=0;
    int arg, rv = 0;
    va_list ap;
    char buf[40];

/*
** The only functions we support at the moment are F_GETFL and F_SETFL, and
** they are inter-twined, so read on.
**
** The only time we seem to use F_SETFL is to set the non-blocking option
** (sometime known as NONBLOCK or NBIO) on a socket. Before this call is
** made to fcntl, the code always does a F_GETFL call to get the current
** flags so that non-blocking can be OR'd in for the set call. Since this
** is the only time we use F_GETFL and since there are no other options,
** we can make our F_GETFL routine just return zero.
**
** Yes, there are some major assumptions here, and it any of the above turns
** out not to be true, then we'll have to rethink some of this.
**
*/

    va_start(ap,cmd);

    switch(cmd) {
        case F_SETFL:
	    arg=va_arg(ap,int);
            if (arg == O_NONBLOCK) {
                rv=ioctl(fildes, FIONBIO, (void *) &one);
                handled=1;
            }
	    else {
                if (arg == 0) {
		    rv=ioctl(fildes, FIONBIO, (void *) &zero);
                    handled=1;
		}
            }
            break;
        case F_GETFL:
	    /* Big assumption here. See comments above */
            rv=0;
            handled=1;
            break;
        case F_DUPFD: 
            rv=dup(fildes);
            handled=1;
            break;
        case F_SETFD:
	    arg=va_arg(ap,int);
	    if (arg==1) {
		rv=fcntl(fildes, cmd, arg);
		handled=1;
	    }
            break;
        case F_GETFD:
	    rv=fcntl(fildes, cmd);
	    handled=1;
            break;
        default:
            break;
    }

#ifdef DEBUG
    if (!handled) {
    switch(cmd) {
        case F_GETFD:
            sprintf(buf,"GETFD");
            break;
        case F_SETFD:
	    arg=va_arg(ap,int);
	    sprintf(buf,"SETFD, arg=%d", arg);
            break;
        case F_GETFL:
            sprintf(buf,"GETFL");
            break;
        case F_SETFL:
            sprintf(buf,"SETFL");
            break;
        case F_GETLK:
            sprintf(buf,"GETLK");
            break;
        case F_SETLK:
            sprintf(buf,"SETLK");
            break;
        case F_SETLKW:
            sprintf(buf,"SETLKW");
            break;
        default:
            sprintf(buf,"?????");
            break;
    }
    GENERIC_TRACE("ALERT - VMSfcntl unhandled request - F_", buf, "", "");
    }
#endif

#ifdef DEBUG_trace_good_ones
    if (handled) {
    switch(cmd) {
        case F_SETFL:
	    printf("FCNTL(SETFL,%d,%d)=%d\n",fildes,arg,rv);
            break;
        case F_GETFL:
	    printf("FCNTL(GETFL,%d)=%d\n",fildes,rv);
            break;
        case F_DUPFD: 
	    printf("FCNTL(DUPFD,%d)=%d\n",fildes,rv);
            break;
        case F_GETFD:
	    printf("FCNTL(GETFD,%d)=%d\n",fildes,rv);
            break;
        case F_SETFD:
	    printf("FCNTL(SETFD,%d,%d)=%d\n",fildes,arg,rv);
            break;
    }
    }
#endif



va_end(ap);

return (rv);

}
