/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use,   *
 * modify, and distribute the following source code (the "Software")     *
 * on any number of computers. You may use the Software as part of       *
 * creating a software program or product intended for commercial or     *
 * non-commercial distribution in machine-readable source code, binary,  *
 * or executable formats. You may distribute the Software as             *
 * machine-readable source code provided this license is not removed     *
 * from the Software and any modifications are conspicuously indicated.  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not remove any copyright, trademark, or other proprietary     *
 * notices from the Software or the associated  documentation.           *
 * You are responsible for compliance with all applicable export or      *
 * re-export control laws and regulations if you export the Software.    *
 * This license is governed by and is to be construed under the laws     *
 * of the State of Texas.                                                *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */

#include "vms_jackets.h"
#include <errno.h>
#include <file.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
/**********************************************************************/
/*  Jacket routines to handle VMS and decc limits                     */
/*                                                                    */
/*  Routine: GENERIC_FOPEN_JACKET                                     */
/*                                                                    */
/*  Jacket for: fopen                                                 */
/*                                                                    */
/*  Reason:  vms has limited like, directory names with .dir extension*/
/*           or 39.39 file limits, hidden files, files with more than */
/*           than one dot.  treat all files are stm/lf                */ 
/*  Fix:     39.39 fix just trunc the filename                        */
/*           hidden files, if '.' is the first character remove it.   */
/*           more than one '.' replace them with '_''s                */
/**********************************************************************/

FILE *decc$fopen(const char *file_spec,const char *a_mode,...);

FILE *GENERIC_FOPEN_JACKET(const char *file_spec, const char *a_mode,...)
{
    char temp_string[TEMP_STRING_SIZE];
    char *new_filename;
    int status;

    new_filename = GENERIC_VMS_Filename_parser((char *)file_spec,
                                        temp_string,
					GENERIC_K_STRING_TYPE_FILE,
					&status);
    GENERIC_TRACE("fopen_Jacket: Mapped:", file_spec, "To:", new_filename);

    /*
    ** Mode "wb+" on OpenVMS always creates a new version of the file. This
    ** is a problem if another thread/process already has the file open
    ** and is expected to see these changes. To solve this, turn any "wb+"
    ** into an "rb+" and then truncate the file to zero bytes.
    */
    if (!strcmp(a_mode,"wb+")) {
	FILE *f;
	int rv;
    	f=decc$fopen(new_filename, "rb+", "ctx=stm", "shr=get,put");
	if (f) {
	    rv=ftruncate(fileno(f),0);
	    if (rv)
		printf("***fopen problem with truncate\n");
	    rv=fflush(f);
	    if (rv)
		printf("***fopen problem with fflush\n");
	return f;
	}
	else {
	    /*
	    ** The fopen failed. It it failed because of "file not found"
	    ** then drop through and try to open the file in the usual
	    ** manner, otherwise return with whatever the error is.
	    */
	    if (errno != ENOENT)
		return NULL;
	}
    }

    return decc$fopen(new_filename, a_mode, "ctx=stm", "shr=get,put");
}
