/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use,   *
 * modify, and distribute the following source code (the "Software")     *
 * on any number of computers. You may use the Software as part of       *
 * creating a software program or product intended for commercial or     *
 * non-commercial distribution in machine-readable source code, binary,  *
 * or executable formats. You may distribute the Software as             *
 * machine-readable source code provided this license is not removed     *
 * from the Software and any modifications are conspicuously indicated.  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not remove any copyright, trademark, or other proprietary     *
 * notices from the Software or the associated  documentation.           *
 * You are responsible for compliance with all applicable export or      *
 * re-export control laws and regulations if you export the Software.    *
 * This license is governed by and is to be construed under the laws     *
 * of the State of Texas.                                                *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */

#include "vms_jackets.h"
#include <stdio.h>
#include <file.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>

/**********************************************************************/
/*  Jacket routines to handle VMS and decc limits                     */
/*                                                                    */
/*  Routine: GENERIC_OPEN_JACKET                                      */
/*                                                                    */
/*  Jacket for: open                                                  */
/**********************************************************************/

int GENERIC_open_mode = -1;
int GENERIC_open_do_umask = 1;

void init_mode() {
    /*
    ** If you don't use umask, you get OpenVMS RMS default protections.
    ** See open in DEC C RTL Ref for OpenVMS 
    **
    ** An open in UNIX allows multiple readers and writers by default. On
    ** OpenVMS we have to ask nicely to get that functionality.
    */
	/* init first time here*/
	/* mode>10 means DON'T do umask trick. Forces all prots to default */
	char *temp_string;
	temp_string = getenv(VMS_PREFIX "FILE_OPEN_MODE");
	if (temp_string) {
	    GENERIC_open_mode = atoi(temp_string);
	    if (GENERIC_open_mode > 9) {
		GENERIC_open_mode -= 10;
		GENERIC_open_do_umask = 0; /* Don't do umask calls */
	    }
	}
	else
	    GENERIC_open_mode = 0;
}

int GENERIC_OPEN_JACKET(const char *file_spec, int flags, mode_t mode)
{
    char *base;
    int ret_int,status, rv;
    mode_t PrevMask, my_mode;
    char temp_string[TEMP_STRING_SIZE];
    char *new_filename;
 
    new_filename = GENERIC_VMS_Filename_parser((char *)file_spec,
					            temp_string,
						    GENERIC_K_STRING_TYPE_FILE,
					            &status);
    GENERIC_TRACE("open_Jacket: Mapped:", file_spec, "To:", new_filename);


    if (GENERIC_open_mode == -1)
	init_mode();

    if (GENERIC_open_do_umask) {
        my_mode = mode; /* Use requested protection */
	PrevMask = umask(0000);
    }
    else
        my_mode = 0777; /* Force default protection */

    if (!GENERIC_open_mode) /* standard open*/
	rv = open(new_filename, flags, my_mode,"shr=get,put");
    else
        if (GENERIC_open_mode==1) /* add stm context to every file */
	    rv = open(new_filename, flags, my_mode, "shr=get,put", "ctx=stm");
	else if (GENERIC_open_mode==2) /* add sharing context to every file */
	    rv = open(new_filename,flags,my_mode, "ctx=rec", "shr=get,put", "rop=rea");
	else if (GENERIC_open_mode==3) /* no sharing */
	    rv = open(new_filename,flags,my_mode);
	else
	    rv = open(new_filename,flags,my_mode,"shr=get,put");

    if (GENERIC_open_do_umask)
	umask(PrevMask);

    return(rv);
}

int GENERIC_OPEN_SHARED_JACKET(const char *file_spec, int flags, mode_t mode)
{
    int status, rv;
    mode_t PrevMask;
    char temp_string[TEMP_STRING_SIZE];
    char *new_filename;
 
    new_filename = GENERIC_VMS_Filename_parser((char *)file_spec,
					            temp_string,
						    GENERIC_K_STRING_TYPE_FILE,
					            &status);
    GENERIC_TRACE("open_shared_Jacket: Mapped:", file_spec,
			"To:", new_filename);

    if (GENERIC_open_mode == -1)
	init_mode();

    if (GENERIC_open_do_umask) {
	PrevMask = umask(0000);
	rv = open(new_filename, flags, mode, "ctx=rec", "shr=get,put,upd" );
	umask(PrevMask);
    }
    else
	/* Force default protection */
	rv = open(new_filename, flags, 0777, "ctx=rec", "shr=get,put,upd" );

    return(rv);
}

/*
** This is an Open Jacket similar to that used by Java for opening
** "user" files. It is needed because Java has to do some funky stuff to
** correctly support some byte mode operations. 
*/

int GENERIC_OPEN_JAVA_JACKET(const char *file_spec, int flags, mode_t mode)
{
    char *base;
    int ret_int,status,rv;
    mode_t PrevMask, my_mode;
    char temp_string[TEMP_STRING_SIZE];
    char *new_filename;
 
    new_filename = GENERIC_VMS_Filename_parser((char *)file_spec,
					            temp_string,
						    GENERIC_K_STRING_TYPE_FILE,
					            &status);
    GENERIC_TRACE("java_open_Jacket: Mapped:", file_spec, "To:", new_filename);

    if (GENERIC_open_mode == -1)
	init_mode();

    if (GENERIC_open_do_umask) {
        my_mode = mode; /* Use requested protection */
	PrevMask = umask(0000);
    }
    else
        my_mode = 0777; /* Force default protection */

    if (!GENERIC_open_mode) /* standard open*/
	rv=open(new_filename,flags,mode);
    else
	if (GENERIC_open_mode==1) /* add stm context to every file */
		rv=open(new_filename,flags,my_mode,"ctx=stm","ctx=rec");
	else if (GENERIC_open_mode==2) /* add sharing context to every file */
		rv=open(new_filename,flags,my_mode,"shr=get,put,upi", "rop=rea");
	else if (GENERIC_open_mode==3) /* add sharing context to every file */
		{
		int reopen_share_check=1;
                while (reopen_share_check)
                {
		ret_int = 
		    open(new_filename,flags,my_mode,"shr=get,put,upi", "rop=rea");
		if (ret_int!= -1)
			{
			GENERIC_FILE_SHARE_REG(ret_int,my_mode);
			if (!GENERIC_FILE_SHARE_CHECK(ret_int)) {
				reopen_share_check=0; /* out of here */
				rv=ret_int; /*everything is good, 
						out of here*/
				break;
			}
			GENERIC_FILE_SHARE_UNREG(ret_int);
			close(ret_int);
			/* didn't clean reopen_share_check so it will
				reopen the file */
			}
		else {
			reopen_share_check=0; /* open failed out of here */
			rv=ret_int;
		     }
		}; /* while */
		}
	else
        /* default */
	    rv=open(new_filename,flags,my_mode);
    if (GENERIC_open_do_umask)
	umask(PrevMask);
    return(rv);
}
